@@ -28,6 +28,8 @@
 (require 'mu4e-utils)    ;; utility functions
 (require 'mu4e-context)  ;; the context
 
+(eval-when-compile
+  (require 'cl))
 
 (defconst mu4e~main-buffer-name " *mu4e-main*"
   "*internal* Name of the mu4e main view buffer.")
@@ -52,7 +54,7 @@
     ;; for terminal users
     (define-key map  (kbd "C-c C-u") 'mu4e-update-mail-and-index)
 
-    (define-key map "S" 'mu4e-interrupt-update-mail)
+    (define-key map "S" 'mu4e-kill-update-mail)
     (define-key map  (kbd "C-S-u") 'mu4e-update-mail-and-index)
     (define-key map ";" 'mu4e-context-switch)
 
@@ -87,31 +89,32 @@ when STR is clicked (using RET or mouse-2); if FUNC-OR-SHORTCUT is
 a string, execute the corresponding keyboard action when it is
 clicked."
   (let ((newstr
-         (replace-regexp-in-string
-          "\\[\\(..?\\)\\]"
-          (lambda(m)
-            (format "[%s]"
-                    (propertize (match-string 1 m) 'face 'mu4e-highlight-face)))
-          str))
-        (map (make-sparse-keymap))
-        (func (if (functionp func-or-shortcut)
-                  func-or-shortcut
-                (if (stringp func-or-shortcut)
-                    (lexical-let ((macro func-or-shortcut))
-                      (lambda()(interactive)
-                        (execute-kbd-macro macro)))))))
+	 (replace-regexp-in-string
+	  "\\[\\(..?\\)\\]"
+	  (lambda(m)
+	    (format "[%s]"
+		    (propertize (match-string 1 m) 'face 'mu4e-highlight-face)))
+	  str))
+	(map (make-sparse-keymap))
+	(func (if (functionp func-or-shortcut)
+		  func-or-shortcut
+		(if (stringp func-or-shortcut)
+		    (lexical-let ((macro func-or-shortcut))
+		      (lambda()(interactive)
+			(execute-kbd-macro macro)))))))
     (define-key map [mouse-2] func)
     (define-key map (kbd "RET") func)
     (put-text-property 0 (length newstr) 'keymap map newstr)
     (put-text-property (string-match "\\[.+$" newstr)
-                       (- (length newstr) 1) 'mouse-face 'highlight newstr) newstr))
+      (- (length newstr) 1) 'mouse-face 'highlight newstr)
+    newstr))
 
 ;; NEW
 ;; This is the old `mu4e~main-view' function but without
 ;; buffer switching at the end.
 (defun mu4e~main-view-real (ignore-auto noconfirm)
   (let ((buf (get-buffer-create mu4e~main-buffer-name))
-        (inhibit-read-only t))
+	(inhibit-read-only t))
     (with-current-buffer buf
       (erase-buffer)
       (insert
@@ -123,21 +126,25 @@ clicked."
        ;; crypto support, a big G when there's Guile support
        " "
        (propertize
-        (concat
-         (when (plist-get mu4e~server-props :crypto) "C")
-         (when (plist-get mu4e~server-props :guile)  "G"))
+	(concat
+	  (when (plist-get mu4e~server-props :crypto) "C")
+	  (when (plist-get mu4e~server-props :guile)  "G")
+	  (when (plist-get mu4e~server-props :mux)  "X"))
 	 'face 'mu4e-title-face)
-	
+
        "\n\n"
        (propertize "  Basics\n\n" 'face 'mu4e-title-face)
-       (mu4e~main-action-str "\t* [j]ump to some maildir\n" 'mu4e-jump-to-maildir)
-       (mu4e~main-action-str "\t* enter a [s]earch query\n" 'mu4e-search)
-       (mu4e~main-action-str "\t* [C]ompose a new message\n" 'mu4e-compose-new)
+	(mu4e~main-action-str
+	  "\t* [j]ump to some maildir\n" 'mu4e-jump-to-maildir)
+	(mu4e~main-action-str
+	  "\t* enter a [s]earch query\n" 'mu4e-search)
+	(mu4e~main-action-str
+	  "\t* [C]ompose a new message\n" 'mu4e-compose-new)
        "\n"
        (propertize "  Bookmarks\n\n" 'face 'mu4e-title-face)
        ;; TODO: it's a bit uncool to hard-code the "b" shortcut...
        (mapconcat
-        (lambda (bm)
+	(lambda (bm)
 	  (mu4e~main-action-str
 	    (concat "\t* [b" (make-string 1 (mu4e-bookmark-key bm)) "] "
 	      (mu4e-bookmark-name bm))
@@ -146,15 +153,15 @@ clicked."
        "\n\n"
        (propertize "  Misc\n\n" 'face 'mu4e-title-face)
 
-	(mu4e~main-action-str "\t* [;]Switch focus\n" 'mu4e-context-switch)
-	
+	(mu4e~main-action-str "\t* [;]Switch context\n" 'mu4e-context-switch)
+
 	(mu4e~main-action-str "\t* [U]pdate email & database\n"
 	  'mu4e-update-mail-and-index)
 
 	;; show the queue functions if `smtpmail-queue-dir' is defined
 	(if (file-directory-p smtpmail-queue-dir)
 	  (mu4e~main-view-queue)
-         "")
+	 "")
 	"\n"
 	(mu4e~main-action-str "\t* [N]ews\n" 'mu4e-news)
 	(mu4e~main-action-str "\t* [A]bout mu4e\n" 'mu4e-about)
@@ -167,37 +174,41 @@ clicked."
   "Display queue-related actions in the main view."
   (concat
    (mu4e~main-action-str "\t* toggle [m]ail sending mode "
-                         'mu4e~main-toggle-mail-sending-mode)
+			 'mu4e~main-toggle-mail-sending-mode)
    "(currently "
    (propertize (if smtpmail-queue-mail "queued" "direct")
-               'face 'mu4e-header-key-face)
+	       'face 'mu4e-header-key-face)
    ")\n"
    (let ((queue-size (mu4e~main-queue-size)))
      (if (zerop queue-size)
-         ""
+	 ""
        (mu4e~main-action-str
-        (format "\t* [f]lush %s queued %s\n"
-                (propertize (int-to-string queue-size)
-                            'face 'mu4e-header-key-face)
-                (if (> queue-size 1) "mails" "mail"))
-        'smtpmail-send-queued-mail)))))
+	(format "\t* [f]lush %s queued %s\n"
+		(propertize (int-to-string queue-size)
+			    'face 'mu4e-header-key-face)
+		(if (> queue-size 1) "mails" "mail"))
+	'smtpmail-send-queued-mail)))))
 
 (defun mu4e~main-queue-size ()
   "Return, as an int, the number of emails in the queue."
   (condition-case nil
       (with-temp-buffer
-        (insert-file-contents (expand-file-name smtpmail-queue-index-file
-                                                smtpmail-queue-dir))
-        (count-lines (point-min) (point-max)))
+	(insert-file-contents (expand-file-name smtpmail-queue-index-file
+						smtpmail-queue-dir))
+	(count-lines (point-min) (point-max)))
     (error 0)))
 
 (defun mu4e~main-view ()
   "Create the mu4e main-view, and switch to it."
-  (mu4e~main-view-real nil nil)
-  (switch-to-buffer mu4e~main-buffer-name)
-  (goto-char (point-min))
+  (if (eq mu4e-split-view 'single-window)
+      (if (buffer-live-p (mu4e-get-headers-buffer))
+	  (switch-to-buffer (mu4e-get-headers-buffer))
+	(mu4e~main-menu))
+    (mu4e~main-view-real nil nil)
+    (switch-to-buffer mu4e~main-buffer-name)
+    (goto-char (point-min)))
   (add-to-list 'global-mode-string '(:eval (mu4e-context-label))))
-    
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Interactive functions
 ;; NEW
@@ -205,16 +216,44 @@ clicked."
 (defun mu4e~main-toggle-mail-sending-mode ()
   "Toggle sending mail mode, either queued or direct."
   (interactive)
-  (let ((curpos (point)))
-    (unless (file-directory-p smtpmail-queue-dir)
-      (mu4e-error "`smtpmail-queue-dir' does not exist"))
-    (setq smtpmail-queue-mail (not smtpmail-queue-mail))
-    (message
-     (concat "Outgoing mail will now be "
-             (if smtpmail-queue-mail "queued" "sent directly")))
-    (mu4e~main-view-real nil nil)
-    (goto-char curpos)))
+  (unless (file-directory-p smtpmail-queue-dir)
+    (mu4e-error "`smtpmail-queue-dir' does not exist"))
+  (setq smtpmail-queue-mail (not smtpmail-queue-mail))
+  (message (concat "Outgoing mail will now be "
+		   (if smtpmail-queue-mail "queued" "sent directly")))
+  (unless (eq mu4e-split-view 'single-window)
+    (let ((curpos (point)))
+      (mu4e~main-view-real nil nil)
+      (goto-char curpos))))
 
+(defun mu4e~main-menu ()
+  "mu4e main view in the minibuffer."
+  (interactive)
+  (let ((key
+	  (read-key
+	   (mu4e-format
+	    "%s"
+	    (concat
+	     (mu4e~main-action-str "[j]ump " 'mu4e-jump-to-maildir)
+	     (mu4e~main-action-str "[s]earch " 'mu4e-search)
+	     (mu4e~main-action-str "[C]ompose " 'mu4e-compose-new)
+	     (mu4e~main-action-str "[b]ookmarks " 'mu4e-headers-search-bookmark)
+	     (mu4e~main-action-str "[;]Switch context " 'mu4e-context-switch)
+	     (mu4e~main-action-str "[U]pdate " 'mu4e-update-mail-and-index)
+	     (mu4e~main-action-str "[N]ews " 'mu4e-news)
+	     (mu4e~main-action-str "[A]bout " 'mu4e-about)
+	     (mu4e~main-action-str "[H]elp " 'mu4e-display-manual))))))
+    (unless (member key '(?\C-g ?\C-\[))
+      (let ((mu4e-command (lookup-key mu4e-main-mode-map (string key) t)))
+	(if mu4e-command
+	    (condition-case err
+		(let ((mu4e-hide-index-messages t))
+		  (call-interactively mu4e-command))
+	      (error (when (cadr err) (message (cadr err)))))
+	  (message (mu4e-format "key %s not bound to a command" (string key))))
+	(when (or (not mu4e-command) (eq mu4e-command 'mu4e-context-switch))
+	  (sit-for 1)
+	  (mu4e~main-menu))))))
 
 ;; (progn
 ;;   (define-key mu4e-compose-mode-map (kbd "C-c m") 'mu4e~main-toggle-mail-sending-mode)
